﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.ServiceModel.Syndication;
using System.Threading.Tasks;
using System.Xml;

namespace GUI
{
    class FeedHandler
    {
        public static Feed GetFeedInformation(String url)
        {
            Feed feed = new Feed { URL = url, Title = url };

            try
            {
                XmlReader reader = XmlReader.Create(url);
                SyndicationFeed feedSynd = SyndicationFeed.Load(reader);
                reader.Close();

                feed.Title = feedSynd.Title.Text;
                feed.Description = feedSynd.Description.Text;

                String feedUrl = null;
                foreach (SyndicationLink li in feedSynd.Links)
                {
                    if (li.RelationshipType == "alternate")
                        feedUrl = li.Uri.AbsoluteUri;
                }

                feed.Favicon = FaviconParser.Load(feedUrl);
                if (String.IsNullOrEmpty(feed.Favicon))
                {
                    feed.Favicon = FaviconParser.Load(url);
                }

                using (WebClient client = new WebClient())
                {
                    feed.Image = Convert.ToBase64String(client.DownloadData(feedSynd.ImageUrl));
                }
            }
            catch (Exception) { }

            return feed;
        }

        public static int UpdateFeed(Feed feed)
        {
            int newItems = 0;

            try
            {
                IList<String> ids = (from b in feed.FeedItem
                                     select b.OriginalID)
                          .ToList();

                XmlReader reader = XmlReader.Create(feed.URL);
                SyndicationFeed feedSynd = SyndicationFeed.Load(reader);
                reader.Close();

                foreach (SyndicationItem item in feedSynd.Items)
                {
                    if (!ids.Contains(item.Id))
                    {
                        newItems++;
                        FeedItem feedItem = new FeedItem();
                        feedItem.OriginalID = item.Id;
                        feedItem.Title = item.Title.Text;
                        feedItem.Summary = item.Summary.Text;
                        feedItem.PublishDate = item.PublishDate.DateTime;
                        feed.FeedItem.Add(feedItem);
                    }
                }
            }
            catch (Exception e) {
                Console.WriteLine(e.Message);
                Console.WriteLine(e.StackTrace);
            }

            return newItems;
        }

        public static async Task<int> UpdateFeedAsync(Feed feed)
        {
            Task<int> task = new Task<int>(() => UpdateFeed(feed), TaskCreationOptions.LongRunning);
            task.Start();
            return await task;
        }
    }
}
