﻿using System;
using System.ComponentModel;
using System.Data.Entity;
using System.IO;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Input;
using System.Windows.Media.Imaging;

namespace GUI
{
    public partial class MainWindow : Window
    {
        private GridViewColumnHeader _feedItemListCurrentSelection = null;
        private ListSortDirection _feedItemListSortDirection = ListSortDirection.Ascending;

        public MainWindow()
        {
            //Database.SetInitializer<DatabaseContainer>(new DropCreateDatabaseAlways<DatabaseContainer>());
            Database.SetInitializer<DatabaseContainer>(new CreateDatabaseIfNotExists<DatabaseContainer>());

            InitializeComponent();
            this.DataContext = new MainViewModel();

            /*x.AddFeed("http://www.engadget.com/rss.xml");
            x.AddFeed("http://diepresse.com/rss/home");
            x.AddFeed("http://feeds.autoblog.com/weblogsinc/autoblog");
            x.AddFeed("http://feeds.feedburner.com/weburbanist");
            x.AddFeed("http://www.chicagobears.com/cda-web/rss-module.htm?tagName=News");
            x.AddFeed("http://feeds.macrumors.com/MacRumors-All");
            x.AddFeed("http://derstandard.at/?page=rss");
            x.AddFeed("http://xkcd.com/rss.xml");
            x.AddFeed("http://www.heise.de/newsticker/heise.rdf");
            x.AddFeed("http://www.autoedizione.com/feed/");
            x.AddFeed("http://www.petrolicious.com/feed/rss");*/
        }

        private void FeedItemsList_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            FeedItem selected = (FeedItem)FeedItemsList.SelectedItem;

            if (selected != null)
            {
                if (!String.IsNullOrEmpty(selected.Summary)) {
                    FeedItemContent.NavigateToString(string.Format("<HTML><HEAD><meta http-equiv='Content-Type' content='text/html;charset=UTF-8'></HEAD><BODY>{0}</BODY></HTML>", selected.Summary));
                }  
            }
            else
            {
                FeedItemContent.Source = null;
            }
        }

        private void AddButton_Click(object sender, RoutedEventArgs e)
        {
            AddFeed af = new AddFeed();
            af.DataContext = DataContext;
            af.ShowDialog();
        }

        private void Window_PreviewKeyDown(object sender, KeyEventArgs e)
        {
            switch (e.Key)
            {
                case Key.Left:
                    if (!FeedList.Items.IsEmpty)
                    {
                        e.Handled = true;
                        int selectedIndex = Math.Max(FeedList.SelectedIndex, 0);
                        FeedList.SelectedIndex = selectedIndex;
                        var selectedItem = FeedList.ItemContainerGenerator.ContainerFromIndex(selectedIndex) as IInputElement;
                        Keyboard.Focus(selectedItem);
                    }
                    break;
                case Key.Right:
                    if (!FeedItemsList.Items.IsEmpty)
                    {
                        e.Handled = true;
                        int selectedIndex = Math.Max(FeedItemsList.SelectedIndex, 0);
                        FeedItemsList.SelectedIndex = selectedIndex;
                        var selectedItem = FeedItemsList.ItemContainerGenerator.ContainerFromIndex(selectedIndex) as IInputElement;
                        Keyboard.Focus(selectedItem);
                        FeedItemsList_SelectionChanged(null, null);
                    }
                    break;
            }
        }

        private void FeedItemList_HeaderClicked(object sender, RoutedEventArgs e)
        {
            GridViewColumnHeader headerClicked = e.OriginalSource as GridViewColumnHeader;
            ListSortDirection direction;

            if (headerClicked != null)
            {
                if (headerClicked.Role != GridViewColumnHeaderRole.Padding)
                {
                    if (headerClicked != _feedItemListCurrentSelection)
                    {
                        direction = ListSortDirection.Ascending;
                    }
                    else
                    {
                        if (_feedItemListSortDirection == ListSortDirection.Ascending)
                        {
                            direction = ListSortDirection.Descending;
                        }
                        else
                        {
                            direction = ListSortDirection.Ascending;
                        }
                    }

                    System.Windows.Data.Binding binding = headerClicked.Column.DisplayMemberBinding as System.Windows.Data.Binding;
                    string header = binding.Path.Path;

                    Sort(header, direction);

                    if (direction == ListSortDirection.Ascending)
                    {
                        headerClicked.Column.HeaderTemplate = Resources["HeaderArrowUp"] as DataTemplate;
                    }
                    else
                    {
                        headerClicked.Column.HeaderTemplate = Resources["HeaderArrowDown"] as DataTemplate;
                    }


                    if (_feedItemListCurrentSelection != null && _feedItemListCurrentSelection != headerClicked)
                    {
                        _feedItemListCurrentSelection.Column.HeaderTemplate = null;
                    }

                    _feedItemListCurrentSelection = headerClicked;
                    _feedItemListSortDirection = direction;
                }
            }
        }

        private void Sort(string sortBy, ListSortDirection direction)
        {
            ICollectionView dataView = CollectionViewSource.GetDefaultView(FeedItemsList.ItemsSource);
            dataView.SortDescriptions.Clear();
            SortDescription sd = new SortDescription(sortBy, direction);
            dataView.SortDescriptions.Add(sd);
            dataView.Refresh();
        }

        private void MainWndw_Loaded(object sender, RoutedEventArgs e)
        {
            ICollectionView dataView = CollectionViewSource.GetDefaultView(FeedList.ItemsSource);
            SortDescription sd = new SortDescription("Title", ListSortDirection.Ascending);
            dataView.SortDescriptions.Add(sd);
            dataView.Refresh();
        }

    }
}
